const { v4: uuidv4 } = require("uuid");

const updaterModel = require("../model/updaterModel");
const updaterModelCasher = require('../model/updaterModelCasher');
const GeraNovoCodigo = require("../modules/NovoCodigoModule");
const OracleService = require("./OracleService");
const Base64 = require("../functions/Base64");
const ServicesMonitoring = require('../helpers/ServicesMonitoring');
const SystemLogger = require("../logs/SystemLogger");

class UpdaterService {
  updaterLog = {};

  async getHistorySeq(CDCAIXA) {
    return CDCAIXA ? GeraNovoCodigo.prxUpdaterHistoricoCx() : GeraNovoCodigo.prxUpdaterHistorico();
  }

  async getEventSeq(CDCAIXA) {
    return CDCAIXA ? GeraNovoCodigo.prxUpdaterEventoCx() : GeraNovoCodigo.prxUpdaterEvento();
  }

  getRequestIp(req) {
    if (process.env.NODE_ENV === 'development') {
      return 'Ambiente de Desenvolvimento';
    }

    return req.headers['x-forwarded-for'] || req.socket.remoteAddress || `IP não encontrado na requisição`;
  }

  async insertEventTable(data) {
    const {
      CDFILIAL,
      CDCAIXA = null,
      CDOPERADOR,
      DEVOPSID,
      PRODFILIID,
      CURRENTVERSION,
      INSTALLVERSION,
      CREATENEWVERSION,
      IDSTEVENTO
    } = data;

    let response = {
      error: true,
      message: "SYSTEM_LOGGED_EVENT",
    };

    let params = {
      P_CDFILIAL: CDFILIAL,
      P_CDCAIXA: CDCAIXA,
      P_CDPRODUCTID: DEVOPSID,
    };

    const model = CDCAIXA ? updaterModelCasher : updaterModel;
    const unitEvents = await model.getEvents(params);

    if (unitEvents.rows.length > 0) return response;
    
    const eventSeq = await this.getEventSeq(CDCAIXA);
    const uuid = uuidv4();
    const basicParams = {
      P_CDFILIAL: CDFILIAL,
      P_CDCAIXA: CDCAIXA,
      P_CDEVENTO: eventSeq,
      P_CDUUID: uuid,
      P_CDPRODUCTID: DEVOPSID,
      P_IDSTEVENTO: IDSTEVENTO,
      P_NRVERSAOATUAL: CURRENTVERSION,
      P_NRVERSAOALVO: INSTALLVERSION,
      P_IDCRIARVERSAO: CREATENEWVERSION,
    };

    await model.insertEvent({...basicParams, P_CDPRODFILIID: PRODFILIID});

    const historySeq = await this.getHistorySeq(CDCAIXA);

    const historyParams = {
      P_CDHISTORICO: historySeq,
      P_CDOPERADOR: CDOPERADOR,
      P_NMPRODUCT: null,
    };

    await model.insertHistory({...basicParams, ...historyParams});

    response = {
      error: false,
      message: "EVENT_REGISTERED_SUCCESSFULLY",
    };

    await OracleService.commit();
    return response;
  }

  async updateEventTable(data) {
    const {
      CDFILIAL,
      CDCAIXA = null,
      CDOPERADOR,
      DEVOPSID,
      CURRENTVERSION,
      INSTALLVERSION,
      CREATENEWVERSION,
      OPERATION
    } = data;

    const model = CDCAIXA ? updaterModelCasher : updaterModel;

    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    const eventId = OPERATION === "INSTALACAO_EM_CURSO" ? "INSTALACAO_PENDENTE" : "REMOCAO_PENDENTE";

    let params = {
      P_CDFILIAL: CDFILIAL,
      P_CDCAIXA: CDCAIXA,
      P_CDPRODUCTID: DEVOPSID,
      P_IDSTEVENTO: eventId,
    };

    const pendingEvents = await model.getEventsByIdAndProduct(params);

    if (pendingEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: OPERATION,
        P_CDEVENTO: pendingEvents.rows[0].CDEVENTO,
      };

      await model.updateEventStatus(params);

      const history = await model.getEventHistory(params);

      if (history.rows.length === 0) {

        const historySeq = await this.getHistorySeq(CDCAIXA);

        params = {
          P_CDFILIAL: CDFILIAL,
          P_CDCAIXA: CDCAIXA,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: pendingEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: CDOPERADOR,
          P_CDPRODUCTID: DEVOPSID,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: OPERATION,
          P_NRVERSAOATUAL: CURRENTVERSION,
          P_NRVERSAOALVO: INSTALLVERSION,
          P_IDCRIARVERSAO: CREATENEWVERSION,
        };

        await model.insertHistory(params);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }
      await OracleService.commit();
    }

    return response;
  }

  async finishEventTable(data) {
    const {
      CDFILIAL,
      CDCAIXA = null,
      CDOPERADOR,
      DEVOPSID,
      PRODFILIID,
      CURRENTVERSION,
      INSTALLVERSION,
      CREATENEWVERSION
    } = data;

    const model = CDCAIXA ? updaterModelCasher : updaterModel;

    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    let params = {
      P_CDFILIAL: CDFILIAL,
      P_CDCAIXA: CDCAIXA,
      P_CDPRODUCTID: DEVOPSID,
    };

    const inProgressEvents = await model.getEventsInProgress(params);

    if (inProgressEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: "FINALIZADO",
        P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO,
      };

      await model.updateEventStatus(params);

      const history = await model.getEventHistory(params);

      if (history.rows.length === 0) {
        const historySeq = await this.getHistorySeq(CDCAIXA);

        params = {
          P_CDFILIAL: CDFILIAL,
          P_CDCAIXA: CDCAIXA,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: inProgressEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: CDOPERADOR,
          P_CDPRODUCTID: DEVOPSID,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: "FINALIZADO",
          P_NRVERSAOATUAL: CURRENTVERSION,
          P_NRVERSAOALVO: INSTALLVERSION,
          P_IDCRIARVERSAO: CREATENEWVERSION,
        };

        await model.insertHistory(params);
        await model.updateProdFiliVersion({
          P_ID: PRODFILIID,
          P_NRVERSAO: INSTALLVERSION
        })

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }

      await OracleService.commit();
    }

    return response;
  }

  async cancelUnitEvents(data) {
    const {
      CDFILIAL,
      CDCAIXA = null
    } = data;

    const model = CDCAIXA ? updaterModelCasher : updaterModel;

    const params = {
      P_CDFILIAL: CDFILIAL,
      P_CDCAIXA: CDCAIXA
    }

    await model.cancelUnitEvents(params);

    await OracleService.commit();
  }

  async cancelEventTable(data) {
    const {
      CDFILIAL,
      CDCAIXA,
      CDOPERADOR,
      DEVOPSID,
      CURRENTVERSION,
      INSTALLVERSION,
      CREATENEWVERSION
    } = data;

    const model = CDCAIXA ? updaterModelCasher : updaterModel;

    let response = {
      error: true,
      message: "O evento já foi atualizado no sistema.",
    };

    let params = {
      P_CDFILIAL: CDFILIAL,
      P_CDCAIXA: CDCAIXA,
      P_CDPRODUCTID: DEVOPSID,
    };

    const unitEvents = await model.getEvents(params);

    if (unitEvents.rows.length > 0) {
      const uuid = uuidv4();

      params = {
        P_IDSTEVENTO: "CANCELADO",
        P_CDEVENTO: unitEvents.rows[0].CDEVENTO,
      };

      await model.updateEventStatus(params);

      const history = await model.getEventHistory(params);

      if (history.rows.length === 0) {
        const historySeq = await this.getHistorySeq(CDCAIXA);

        params = {
          P_CDFILIAL: CDFILIAL,
          P_CDCAIXA: CDCAIXA,
          P_CDHISTORICO: historySeq,
          P_CDEVENTO: unitEvents.rows[0].CDEVENTO,
          P_CDUUID: uuid,
          P_CDOPERADOR: CDOPERADOR,
          P_CDPRODUCTID: DEVOPSID,
          P_NMPRODUCT: null,
          P_IDSTEVENTO: "CANCELADO",
          P_NRVERSAOATUAL: CURRENTVERSION,
          P_NRVERSAOALVO: INSTALLVERSION,
          P_IDCRIARVERSAO: CREATENEWVERSION,
        };

        await model.insertHistory(params);

        response = {
          error: false,
          message: "Evento atualizado com sucesso. Histórico criado",
        };
      }
      await OracleService.commit();
    }

    return response;
  }

  async getPendingEvents() {
    return updaterModel.getPendingEvents();
  }

  async getPendingEventsByUnit(CDFILIAL, CDCAIXA) {
    try {
      const params = {
        P_CDFILIAL: CDFILIAL,
        P_CDCAIXA: CDCAIXA,
      };

      const model = CDCAIXA ? updaterModelCasher : updaterModel;
      const pendingEvents = await model.getUpdaterPendingEvents(params);

      return {
        error: false,
        message: 'Nenhum evento pendente de instalação/remoção encontrado.',
        events: pendingEvents.rows
      }
    } catch (error) {
      return {
        error: true,
        message: error,
        events: {}
      }
    }
  }
  
  async checkVersion(row) {
    const model = row.CDCAIXA ? updaterModelCasher : updaterModel;
    const {
      CDFILIAL: P_CDFILIAL,
      CDCAIXA: P_CDCAIXA,
      UPDATERPRODID: P_UPDATERPRODID,
      NRVERSAOATUAL: P_NRVERSAOATUAL
    } = row;
    const params = { P_CDFILIAL, P_CDCAIXA, P_UPDATERPRODID, P_NRVERSAOATUAL };
    const result = await model.getVersion(params);
    return { error: result.rows.length > 0 };
  }
  
  async saveConfig({ config }){
    try {
      if (!config) {
        throw {
          error: true,
          message: 'Codigo da Filial não foi informado'
        };
      }

      const params = {
        P_CDFILIAL: config.environment.CDFILIAL
      };
      
      let configIni = await updaterModel.getConfigIni(params);

      if (configIni.rows.length > 0) {
        return {
          error: false,
          message: `Arquivo config.ini da Filial ${params.P_CDFILIAL} já está salvo na base de dados.`
        };
      };

      const configIniBase64 = Base64.encode(config);
      
      if (!configIniBase64) {
        throw {
          error: true,
          message: 'Não foi possivel converter config.ini da filial para base64.'
        };
      }
      
      params.P_CONFIG = configIniBase64;

      const insertConfig = await updaterModel.insertConfigIni(params);
      
      if (insertConfig.rowsAffected === 0 || insertConfig.outBinds.ID.length === 0) {
        throw {
          error: true,
          message: 'Erro ao inserir as informações do config.ini na base de dados.'
        };
      }

      const operator = await updaterModel.getDefaultOperator(params);

      if (operator.rows.length === 0) {
        throw {
          error: true,
          message: `Não foi possivel buscar o operador/vendedor padrão da filial ${params.P_CDFILIAL}.`
        };
      }

      if (operator.rows[0].CDOPERADOR === null && operator.rows[0].CDVENDEDOR === null) {
        throw {
          error: true,
          message: `Os campos de Operador Padrão e Vendedor Padrão da filial ${params.P_CDFILIAL} não estão preenchidos.`
        };
      }

      const { CDOPERADOR } = operator.rows[0]; 
      const defaultOperator = CDOPERADOR ? CDOPERADOR : await updaterModel.getDefaultOperatorFromSalesman({P_CDVENDEDOR: operator.rows[0].CDVENDEDOR}).then(response => {
        if(response && response.rows && response.rows[0]) {
          let [ firstRow ] = response.rows;
          let { CDOPERADOR } = firstRow;
          return CDOPERADOR;
        }
        return null;
      });

      params.P_CDOPERADOR = defaultOperator;
      params.P_UPDATERCONFIGID = insertConfig.outBinds.ID[0];

      const appendConfigIni = await updaterModel.appendConfigIniHistory(params);

      if (appendConfigIni.rowsAffected === 0) {
        throw {
          error: true,
          message: 'Falha ao inserir na tabela de histórico de alteração do arquivo config.ini.'
        };
      }

      await OracleService.commit();       
      
      return {
        error: false,
        message: `Config.ini da filial ${params.P_CDFILIAL} foi inserido com sucesso.`
      }
    } catch(error) {     
      SystemLogger.error(error);
      await OracleService.rollback();
      
      return error;
    }     
  }

  async getDefaultOperatorFromSalesman(salesmanCode) {
    try {
      params.P_CDVENDEDOR = salesmanCode;
      const salesman = await updaterModel.getDefaultOperatorFromSalesman(params);

      if (salesman.rows.length === 0 || salesman.rows[0].CDOPERADOR !== null) {
        throw {
          error: true,
          message: `O campo de operador padrão deve estar preenchido ou associado a um vendedor padrão para a filial ${params.P_CDFILIAL}.`
        };
      }

      return salesman.rows[0].CDOPERADOR
    } catch (error) {
      throw error;
    }
  }

  async getUnitConfigIniData(params) {
    try {
      const result = await updaterModel.getConfigIniByUnit(params);
  
      if (result.rows.length === 0) {
        throw new Error(`Não existe um arquivo de configuração salvo para a filial ${params.P_CDFILIAL}`);
      }

      return result.rows[0];
    } catch (error) {
      throw error;
    }
  }

  async getConfigIniByUnit(cdFilial) {
    try {
      const params = {
        P_CDFILIAL: cdFilial,
      };

      const result = await this.getUnitConfigIniData(params);

      const decodeConfig = Base64.decode(result.CONFIG);
      
      return {
        error: false,
        message: 'SUCCESS_LOAD_CONFIG_INI',
        data: decodeConfig
      }
    } catch (error) {
      return {
        error: true,
        message: 'FAIL_LOAD_CONFIG_INI'
      }
    }
  }

  async saveConfigIniByUnit(cdFilial, cdOperador, config) {
    try {
      const encodeConfig = Base64.encode(config);
      let result;

      let params = {
        P_CDFILIAL: cdFilial,
        P_CDOPERADOR: cdOperador,
        P_CONFIG: encodeConfig
      }
      
      result = await updaterModel.updateConfigIniByUnit(params);

      if (result.rowsAffected === 0) {
        throw new Error('NO_CONFIG_FILE_FOR_SPECIFIC_UNIT');
      }

      result = await this.getUnitConfigIniData(params);

      params.P_UPDATERCONFIGID = result.ID;

      result = await updaterModel.appendConfigIniHistory(params);
      
      if (result.rowsAffected === 0) {
        throw new Error('FAIL_SAVE_CONFIG_INI_HISTORY');
      }

      await OracleService.commit();
      
      return {
        error: false,
        message: 'SUCCESS_SYNC_CONFIG_INI'
      }
    } catch (error) {
      await OracleService.rollback();
      
      return {
        error: true,
        message: error.message,
        params: { 'CDFILIAL': cdFilial }
      }
    }
  }

  async getConfigHistoryByUnit(cdFilial) {
    try {
      let result;
      const params = {
        P_CDFILIAL: cdFilial,
      };

      result = await this.getUnitConfigIniData(params);
      
      params.P_ID = result.ID;
      
      result = await updaterModel.getConfigIniHistoryByUnit(params);
      
      return {
        error: false,
        message: 'SUCCESS_LOAD_UPDATE_HISTORY_CONFIG',
        data: result.rows
      }
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }

  handleConfigStructure( configString, configValues = {} ) {
    if(!configString) return {};
    if(Base64.isValid(configString)) {
      return {...Base64.decode(configString), ...configValues};
    }
    return configString.split('|').reduce((acc, key) => {
      acc[key] = (key in configValues) ? configValues[key] : '';
      return acc;
    }, {});
  }

  hasEmptyParams( config ){
    return Object.values(config).some( value => !value);
  }

  setMessageFillEmptyParams( response ){
    if(this.hasEmptyParams(response.data)) {
      response.message = "Preencha os parâmetros do projeto antes de continuar instalação.";
    }
  }

  async getLatestConfig( params ){
    const updaterProdConfigLastUpdate = await updaterModel.getUpdaterProdConfigLastUpdate(params);
    if(updaterProdConfigLastUpdate.rows.length){
      const [ LATEST_VERSION_CONFIG ] = updaterProdConfigLastUpdate.rows;
      const { CONFIG } = LATEST_VERSION_CONFIG;
      return CONFIG;
    }
    return null;
  }

  async getProdConfig(cdProductId, nrVersao) {
    try {
      const params = {
        CDPRODUCTID: cdProductId,
        NRVERSAO: nrVersao
      };

      if (!params.CDPRODUCTID) {
        throw new Error('Codigo do produto não foi informado.'); 
      }
        
      let updaterProdConfig = await updaterModel.getUpdaterProdConfig(params);
      let response = { error: false, data: {} };
      if (updaterProdConfig.rows.length === 0) {
        let CONFIG = await this.getLatestConfig(params);
        await updaterModel.insertUpdaterProdConfig({...params, CONFIG}).catch( error => {
          SystemLogger.error(error);
          response.error = true;
          response.message = 'Não foi encontrado um registro de configuração para o produto na base de dados.';
          response.errorInfo = error;
        });

        response.data = this.handleConfigStructure(CONFIG);
        if(!response.error) {
          await OracleService.commit();
          this.setMessageFillEmptyParams(response);
        }
        return response;
      }

      let config = updaterProdConfig.rows[0].CONFIG;
      let defaultValues = {};
      let latestConfig = null;
      if(!config) return response;
      if(!Base64.isValid(config)) {
        latestConfig = await this.getLatestConfig(params);
      }
      if(Base64.isValid(latestConfig)) {
        defaultValues = Base64.decode(latestConfig);
      }
      response.data = this.handleConfigStructure(config, defaultValues);
      this.setMessageFillEmptyParams(response);
      return response;
    } catch (error) {
      return {
        error: true,
        message: error.message
      }
    }
  }

  resultNewProductConfiguration(config, decode){
    let prodConfigObject = {};          
    
    config.map((element) => {      
      if (element in decode) {      
        prodConfigObject[element] = decode[element];
      } else {
        prodConfigObject[element] = '';
      }
    })  
    return prodConfigObject;   
  }

  async saveProdConfig(CDPRODUCTID, NRVERSAO, CONFIG){
    try {
      const hasEmptyValue = Object.values(CONFIG).some( value => value === '');
      if(hasEmptyValue) throw new Error('Ocorreu um erro ao salvar as informações, tente novamente.');

      const encodedConfig = typeof CONFIG === 'string' ? CONFIG : Base64.encode(CONFIG);

      if (!encodedConfig) {
        throw new Error(`Não foi possivel converter os parâmetros do produto ${CDPRODUCTID} para base64.`);
      }  
    
      const updaterProductConfig = {
        CDPRODUCTID,
        NRVERSAO,
        CONFIG: encodedConfig
      }
      const { rows } = await updaterModel.getUpdaterProdConfig({CDPRODUCTID, NRVERSAO});
      let response = null;
      if(rows && rows.length) {
        response = await updaterModel.setUpdateProdConfig(updaterProductConfig);
      } else {
        response = await updaterModel.insertUpdaterProdConfig(updaterProductConfig);
      }
      
      if (response && response.rowsAffected === 0) {
        throw new Error(`Falha ao salvar parâmetros do produto ${CDPRODUCTID}.`);
      }
    
      await OracleService.commit();

      return {
        error: false,
        message: 'Parâmetros do projeto salvos com sucesso!',
        data: CONFIG
      };
    } catch(error) {    
      await OracleService.rollback();
      
      return {
        error: true,
        message: error.message
      }
    }
  }

  handleAddServicesInUnit(actualServices, servicesToAdd, separator = ';') {
    const hasNewServices = servicesToAdd.some( service => !actualServices.includes(service) );
    if(!hasNewServices) return actualServices.join(separator);
    const allServices = actualServices.concat(servicesToAdd);
    const servicesToSave = [ ...new Set(allServices) ];
    return servicesToSave.join(separator);
  }

  handleRemoveServicesInUnit(actualServices, servicesToRemove, separator = ';') {
    const servicesUpdated = actualServices.filter( service => !servicesToRemove.includes(service) );
    return servicesUpdated.join(separator);
  }

  async handleChangeServiceByUnit(cdFilial, updatedServices, operation) {
    const SUCCESS_RESPONSE = {
      error: false,
      message: 'Monitoramento de serviços atualizado com sucesso!'
    };

    const functionsToHandle = {
      INSTALL: this.handleAddServicesInUnit,
      REMOVE: this.handleRemoveServicesInUnit,
      default: (actualServices) => actualServices,
      ERROR: (error) => {
        SystemLogger.error(error);
        throw new Error('Ocorreram erros ao atualizar os servicos monitorados na unidade');
      }
    }
    try {
      const params = {
        P_CDFILIAL: cdFilial
      };
      
      const actualServices = await this.getServiceMonitorConfigurationByUnit(params).catch(functionsToHandle.ERROR);
      
      const functionToCall = functionsToHandle[operation.toUpperCase()] || functionsToHandle.default;
      const servicesToSave = functionToCall.apply(this, [actualServices.split(';'), updatedServices.split(';')]);

      params.P_MONITORAMENTO = ServicesMonitoring.removeMonitoringParamInconsistencies(servicesToSave);
      
      await updaterModel.updateServiceMonitorConfigurationByUnit(params).catch(functionsToHandle.ERROR);
      await OracleService.commit();
      
      return SUCCESS_RESPONSE;
    } catch (error) {
      SystemLogger.error(error);
      return {
        error: true,
        message: error
      };
    }
  }

  async getServiceMonitorConfigurationByUnit(params) {
    try {
      const result = await updaterModel.getServiceMonitorConfigurationByUnit(params);

      if (result.rows.length === 0) {
        throw "Não foi encontrado um monitoramento de serviços ativo.";
      }

      return result.rows[0].MONITORAMENTO ? result.rows[0].MONITORAMENTO : '';
    } catch (error) {
      throw error;
    }
  }

  async installNewVersion(params) {
    try {    
      if(!params.UPDATERPRODID) throw 'Código do produto não informado.';

      const { rows } = await updaterModel.getUpdateLastRecordProduct(params);

      if(rows.length === 0) throw 'Erro ao efetuar busca do produto.';
     
      if(rows[0].NRVERSAO === params.NRVERSAO) throw 'Essa versão já está instalada.';

      const newVersionObject = {       
        UPDATERPRODID: params.UPDATERPRODID,
        NRVERSAO: params.NRVERSAO,
        DSURL: params.DSURL,
        DSDIRETORIO: rows[0].DSDIRETORIO,        
        IDATIVO: rows[0].IDATIVO,    
        IDDOWNLOADVER: 'N',
        IDAPROVVER: rows[0].IDAPROVVER
      };     

      const { rowsAffected } = await updaterModel.setUpdateProdVersion(newVersionObject);

      if(rowsAffected === 0) throw 'Erro ao inserir informações do produto.';  

      const lastConfig = await this.getLatestConfig({CDPRODUCTID: params.UPDATERPRODID, NRVERSAO: params.NRVERSAO});

      const prodConfig = {
        CDPRODUCTID: params.UPDATERPRODID, 
        NRVERSAO: params.NRVERSAO, 
        CONFIG: lastConfig       
      };

      const resultSet = await updaterModel.insertUpdaterProdConfig(prodConfig);
      
      if(resultSet.rowsAffected === 0) throw 'Erro ao inserir informações do produto.';  

      await OracleService.commit();

      return {
        error: false,
        message: 'As informações foram salvas com sucesso.'
      }
    } catch (error) {
      await OracleService.rollback();
      return {
        error: true,
        message: error
      }
    }
  }

  async removeEventsByUnitAndProduct( params ) {
    try {
      await updaterModel.removeHistoryEvents(params);
      await updaterModel.removeEventsByUnitAndProduct(params);
      
      OracleService.commit();

      return {
        error: false,
        message: ''
      }
    } catch( error ) {
      OracleService.rollback();

      return {
        error: true,
        message: error.message
      }
    }
  }

  async updatePackageURL(params) {
    try {
      const { UPDATERPRODID, NRVERSAO, DSURL, DSDIRETORIO } = params;

      if (!UPDATERPRODID || !NRVERSAO || !DSURL || !DSDIRETORIO) {
        throw 'Missing needed parameters: UPDATERPRODID, NRVERSAO, DSURL, DSDIRETORIO';
      }

      const result = await updaterModel.updatePackageURL(params);

      await OracleService.commit();

      return {
        error: false,
        message: `${result.rowsAffected} registro atualizado.`
      }
    } catch (error) {
      await OracleService.rollback();

      return {
        error: true,
        message: error
      }
    }
  }

  async getProductsVersion(params) {
    const {
      CDFILIAL: P_CDFILIAL,
      CDCAIXA: P_CDCAIXA,
    } = params;
    const model = P_CDCAIXA ? updaterModelCasher : updaterModel;
    return model.getProductsVersion({P_CDFILIAL, P_CDCAIXA});
  }

}

module.exports = new UpdaterService();
